<div style="display: flex; align-items: center;">
    <style>
        .page-limit-label {
            float: left;
        }
        .page-limit-fieldset {
            float: left;
            border: none;
            display: inline;
        }
        .page-limit-fieldset > select {
            min-width: 5em;
        }
    </style>
    <span class="page-limit-label">{{ _("Page limit") }}: </span>
    <fieldset class="page-limit-fieldset">
        <select class="page-limit-select">
            <option value="10">10</option>
            <option value="25">25</option>
            <option value="50" selected>50</option>
            <option value="100">100</option>
            <option value="0">{{ _("All") }}</option>
        </select>
    </fieldset>
</div>