{% extends "base.jinja" %}
{% block main %}
    {% include "includes/address-label.html" %}
    <h1 id="title">{{wallet.name}}</h1>
    {% if tab == 'settings' %}
        {% from 'components/editable_title.jinja' import editable_title %}
        {{ editable_title(wallet.name) }}
    {% endif %}
    {% from 'wallet/components/wallet_menu.jinja' import wallet_menu %}
    {{ wallet_menu(tab, wallet_alias) }}
    <div id="tx-popup" class="hidden"></div>
    {% block content %}
    {% endblock %}
{% endblock %}
